"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _mapper = require("../mapper");

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var buildXhrResponse = function buildXhrResponse(xhr) {
  return {
    data: xhr.response,
    status: xhr.status,
    statusText: xhr.statusText,
    ok: xhr.status >= 200 && xhr.status < 300,
    responseHeaders: xhr.getAllResponseHeaders(),
    getResponseHeader: xhr.getResponseHeader
  };
};

var XhrHttpLayer = function () {
  function XhrHttpLayer() {
    _classCallCheck(this, XhrHttpLayer);

    this.requestsContainer = [];
  }

  _createClass(XhrHttpLayer, [{
    key: "addRequest",
    value: function addRequest(xhr, endPoint) {
      var req = { endPoint: endPoint, xhr: xhr };
      this.requestsContainer = [].concat(_toConsumableArray(this.requestsContainer), [req]);
    }
  }, {
    key: "addMemRequest",
    value: function addMemRequest(timeoutId, endPoint, onAbort) {
      var req = {
        endPoint: endPoint,
        xhr: {
          abort: function abort() {
            clearTimeout(timeoutId);
            onAbort();
          }
        }
      };

      this.requestsContainer = [].concat(_toConsumableArray(this.requestsContainer), [req]);
    }
  }, {
    key: "handleRequest",
    value: function handleRequest(stateDispatcher, request, method) {
      var options = request.options;
      var beforeRequest = options.beforeRequest || function () {};
      var afterResponse = options.afterResponse || function () {};
      var afterError = options.afterError || function () {};
      var afterAbort = options.afterAbort || function () {};
      var parseResponse = options.parseResponse || options.responseParse || function (r) {
        return r.data;
      };
      var parseBody = options.parseBody || options.bodyParse || function (b) {
        return b ? JSON.stringify(b) : b;
      };

      var memResponse = options.memResponse;
      var delay = options.delay;
      var cacheHandler = options.cacheHandler;

      var responseType = options.responseType || "json";
      var mimeType = options.mimeType;

      var cbSuccess = function cbSuccess() {};
      var cbError = function cbError() {};
      var cbAbort = function cbAbort() {};

      if (memResponse !== undefined) {
        var memResponseFn = memResponse;

        beforeRequest(request);
        stateDispatcher(_mapper.FetchStates.FETCH_STARTED);

        if (typeof memResponse !== "function") memResponseFn = function memResponseFn() {
          return memResponse;
        };

        if (delay > 0) {
          var timeoutId = setTimeout(function () {
            var response = memResponseFn(request);
            stateDispatcher(_mapper.FetchStates.FETCH_COMPLETED, response);
            afterResponse(request, response);
            cbSuccess(response);
          }, delay);
          this.addMemRequest(timeoutId, request.endPoint, cbAbort);
        } else {
          var response = memResponseFn(request);
          stateDispatcher(_mapper.FetchStates.FETCH_COMPLETED, response);
          afterResponse(request, response);
          cbSuccess(response);
        }
      } else {
        beforeRequest(request);
        stateDispatcher(_mapper.FetchStates.FETCH_STARTED);
        var cacheResponse = void 0;

        if (cacheHandler) cacheResponse = cacheHandler(request);

        if (cacheResponse !== undefined) {
          stateDispatcher(_mapper.FetchStates.FETCH_COMPLETED, cacheResponse);
          afterResponse(request, cacheResponse);
          cbSuccess(cacheResponse);
        } else {
          var xhr = new XMLHttpRequest();
          this.addRequest(xhr, request.endPoint);

          var handleLoad = function handleLoad() {
            var parsedResponse = parseResponse(buildXhrResponse(xhr));
            stateDispatcher(_mapper.FetchStates.FETCH_COMPLETED, parsedResponse);
            afterResponse(request, parsedResponse);
            cbSuccess(parsedResponse);
          };

          var handleError = function handleError() {
            var parsedResponse = parseResponse(buildXhrResponse(xhr));
            stateDispatcher(_mapper.FetchStates.FETCH_ERROR, parsedResponse);
            afterError(request, parsedResponse);
            cbError(parsedResponse);
          };

          var handleAbort = function handleAbort() {
            stateDispatcher(_mapper.FetchStates.FETCH_CANCELLED);
            afterAbort(request);
            cbAbort();
          };

          xhr.addEventListener('load', handleLoad);
          xhr.addEventListener('error', handleError);
          xhr.addEventListener('abort', handleAbort);

          xhr.open(method, request.fullPath);

          for (var header in request.headers) {
            xhr.setRequestHeader(header, request.headers[header]);
          }xhr.responseType = responseType;

          if (mimeType) xhr.overrideMimeType(mimeType);

          xhr.send(parseBody(request.body));
        }
      }

      return function () {
        var onSuccess = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : cbSuccess;
        var onError = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : cbError;
        var onAbort = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : cbAbort;

        cbSuccess = onSuccess;
        cbError = onError;
        cbAbort = onAbort;
      };
    }
  }, {
    key: "abortRequest",
    value: function abortRequest(endPoint) {
      var aborted = false;

      this.requestsContainer = this.requestsContainer.filter(function (req) {
        if (req.endPoint === endPoint) {
          req.xhr.abort();
          aborted = true;
          return false;
        }
        return true;
      });

      return aborted;
    }
  }, {
    key: "get",
    value: function get(stateDispatcher, request) {
      return this.handleRequest(stateDispatcher, request, _mapper.HttpMethods.GET);
    }
  }, {
    key: "post",
    value: function post(stateDispatcher, request) {
      return this.handleRequest(stateDispatcher, request, _mapper.HttpMethods.POST);
    }
  }, {
    key: "put",
    value: function put(stateDispatcher, request) {
      return this.handleRequest(stateDispatcher, request, _mapper.HttpMethods.PUT);
    }
  }, {
    key: "delete",
    value: function _delete(stateDispatcher, request) {
      return this.handleRequest(stateDispatcher, request, _mapper.HttpMethods.DELETE);
    }
  }, {
    key: "patch",
    value: function patch(stateDispatcher, request) {
      return this.handleRequest(stateDispatcher, request, _mapper.HttpMethods.PATCH);
    }
  }, {
    key: "head",
    value: function head(stateDispatcher, request) {
      return this.handleRequest(stateDispatcher, request, _mapper.HttpMethods.HEAD);
    }
  }]);

  return XhrHttpLayer;
}();

exports.default = XhrHttpLayer;