'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createMapper = createMapper;
exports.addResourceToMapper = addResourceToMapper;
exports.addEndPointToResource = addEndPointToResource;
exports.buildEndPointFunc = buildEndPointFunc;

var _httpMethods = require('./http-methods');

var _httpMethods2 = _interopRequireDefault(_httpMethods);

var _pathBuilder = require('../utils/path-builder');

var _pathBuilder2 = _interopRequireDefault(_pathBuilder);

var _request = require('./request');

var _request2 = _interopRequireDefault(_request);

var _stateDispatcher = require('./state-dispatcher');

var _stateDispatcher2 = _interopRequireDefault(_stateDispatcher);

var _actionBuilder = require('../reducer/action-builder');

var _actionBuilder2 = _interopRequireDefault(_actionBuilder);

var _httpLayers = require('../http-layers');

var _merge = require('../utils/merge');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function createMapper(store, config, httpLayer) {
  var mapper = {
    store: store,
    httpLayer: httpLayer ? httpLayer : new _httpLayers.FetchHttpLayer(),
    host: config.host,
    headers: config.headers,
    options: config.options
  };

  var resources = config.resources;

  resources.forEach(function (resource) {
    addResourceToMapper(mapper, resource);
  });

  return mapper;
}

function addResourceToMapper(mapper, resource) {
  var mResource = {
    _mapper: mapper,
    name: resource.name,
    path: resource.path,
    headers: resource.headers,
    options: resource.options
  };

  var endPoints = resource.endPoints;

  endPoints.forEach(function (endPoint) {
    addEndPointToResource(mResource, endPoint);
  });

  mapper[resource.name] = mResource;
}

function addEndPointToResource(resource, endPoint) {
  var mEndPoint = {
    _resource: resource,
    name: endPoint.name,
    path: endPoint.path,
    headers: endPoint.headers,
    options: endPoint.options,
    method: endPoint.method ? endPoint.method : _httpMethods2.default.GET,
    action: endPoint.action
  };

  var actionBuilder = (0, _actionBuilder2.default)(resource, endPoint);

  resource[endPoint.name] = buildEndPointFunc(mEndPoint);
  resource[endPoint.name].path = mEndPoint.path;
  resource[endPoint.name].headers = mEndPoint.headers;
  resource[endPoint.name].options = mEndPoint.options;
  resource[endPoint.name].method = mEndPoint.method;
  resource[endPoint.name].action = mEndPoint.action;
  resource[endPoint.name].clearState = function () {
    var dispatch = resource._mapper.store.dispatch;

    dispatch({ type: actionBuilder("CLEAR_STATE") });
  };
  resource[endPoint.name].dispatch = function (action) {
    if (action.type) action.type = actionBuilder(action.type);

    resource._mapper.store.dispatch(action);
  };
}

function buildEndPointFunc(mEndPoint) {
  return function (params, body, headers, options) {
    var mapper = mEndPoint._resource._mapper;
    var resource = mEndPoint._resource;
    var httpLayer = mEndPoint._resource._mapper.httpLayer;
    var store = mapper.store;

    var ePath = resource[mEndPoint.name].path;
    var eHeaders = resource[mEndPoint.name].headers;
    var eOptions = resource[mEndPoint.name].options;
    var eAction = resource[mEndPoint.name].action;
    var eMethod = resource[mEndPoint.name].method;

    var fullPath = (0, _pathBuilder2.default)(mapper.host, resource.path, ePath, params);

    if (httpLayer.mergeHeaders) headers = httpLayer.mergeHeaders(mapper.headers, resource.headers, eHeaders, headers);else headers = (0, _merge.mergeHeaders)(mapper.headers, resource.headers, eHeaders, headers);

    if (httpLayer.mergeOptions) options = httpLayer.mergeOptions(mapper.options, resource.options, eOptions, options);else options = (0, _merge.mergeOptions)(mapper.options, resource.options, eOptions, options);

    var request = (0, _request2.default)(fullPath, eMethod, params, headers, body, mapper, resource, resource[mEndPoint.name], options);
    var stateDispatcher = (0, _stateDispatcher2.default)(store, eAction, mEndPoint);

    if (!httpLayer[eMethod]) throw new Error('Current http-layers do not support ' + eMethod + ' method');

    return httpLayer[eMethod](stateDispatcher, request);
  };
}