"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _mapper = require("../mapper");

function buildReducerBuilder(initialData) {
  var defState = {
    isFetching: false,
    isError: false,
    isSuccess: false,
    isCancelled: false,
    cancelledData: null,
    data: initialData || null,
    errorData: null
  };

  return function defReducerBuilder(actionBuilder) {
    var FETCH_STARTED = actionBuilder(_mapper.FetchStates.FETCH_STARTED);
    var FETCH_COMPLETED = actionBuilder(_mapper.FetchStates.FETCH_COMPLETED);
    var FETCH_ERROR = actionBuilder(_mapper.FetchStates.FETCH_ERROR);
    var FETCH_CANCELLED = actionBuilder(_mapper.FetchStates.FETCH_CANCELLED);
    var CLEAR_STATE = actionBuilder("CLEAR_STATE");

    return function reducer() {
      var prevState = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defState;
      var action = arguments[1];

      switch (action.type) {
        case FETCH_STARTED:
          return _extends({}, defState, { isFetching: true });
        case FETCH_COMPLETED:
          return _extends({}, defState, { isSuccess: true, data: action.payload });
        case FETCH_CANCELLED:
          return _extends({}, defState, { isCancelled: true, cancelledData: action.payload });
        case FETCH_ERROR:
          return _extends({}, defState, { isError: true, errorData: action.payload });
        case CLEAR_STATE:
          return defState;
      }
      return prevState;
    };
  };
}

exports.default = buildReducerBuilder;